@@ -290,6 +290,7 @@ class SmallSearchSpaceWarning(HypothesisWarning):
 
 
 CannotProceedScopeT = Literal["verified", "exhausted", "discard_test_case", "other"]
+_valid_cannot_proceed_scopes = CannotProceedScopeT.__args__  # type: ignore
 
 
 class BackendCannotProceed(HypothesisException):
@@ -317,4 +318,9 @@ class BackendCannotProceed(HypothesisException):
     """
 
     def __init__(self, scope: CannotProceedScopeT = "other", /) -> None:
+        if scope not in _valid_cannot_proceed_scopes:
+            raise InvalidArgument(
+                f"Got scope={scope}, but expected one of "
+                f"{_valid_cannot_proceed_scopes!r}"
+            )
         self.scope = scope
