@@ -9,11 +9,12 @@
 # obtain one at https://mozilla.org/MPL/2.0/.
 
 from collections import OrderedDict, abc
+from collections.abc import Sequence
 from copy import copy
+from dataclasses import dataclass
 from datetime import datetime, timedelta
-from typing import Any, List, Optional, Sequence, Set, Union
+from typing import Any, Generic, Union
 
-import attr
 import numpy as np
 import pandas
 
@@ -24,7 +25,6 @@ from hypothesis.errors import InvalidArgument
 from hypothesis.extra import numpy as npst
 from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.coverage import check, check_function
-from hypothesis.internal.reflection import get_pretty_function_description
 from hypothesis.internal.validation import (
     check_type,
     check_valid_interval,
@@ -35,13 +35,9 @@ from hypothesis.strategies._internal.strategies import Ex, check_strategy
 from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 
 try:
-    from pandas.api.types import is_categorical_dtype
+    from pandas.core.arrays.integer import IntegerDtype
 except ImportError:
-
-    def is_categorical_dtype(dt):
-        if isinstance(dt, np.dtype):
-            return False
-        return dt == "category"
+    IntegerDtype = ()
 
 
 def dtype_for_elements_strategy(s):
@@ -59,7 +55,6 @@ def infer_dtype_if_necessary(dtype, values, elements, draw):
 
 @check_function
 def elements_and_dtype(elements, dtype, source=None):
-
     if source is None:
         prefix = ""
     else:
@@ -74,43 +69,69 @@ def elements_and_dtype(elements, dtype, source=None):
                     f"At least one of {prefix}elements or {prefix}dtype must be provided."
                 )
 
-    with check("is_categorical_dtype"):
-        if is_categorical_dtype(dtype):
+    with check("isinstance(dtype, CategoricalDtype)"):
+        if pandas.api.types.CategoricalDtype.is_dtype(dtype):
             raise InvalidArgument(
                 f"{prefix}dtype is categorical, which is currently unsupported"
             )
 
+    if isinstance(dtype, type) and issubclass(dtype, IntegerDtype):
+        raise InvalidArgument(
+            f"Passed {dtype=} is a dtype class, please pass in an instance of this class."
+            "Otherwise it would be treated as dtype=object"
+        )
+
     if isinstance(dtype, type) and np.dtype(dtype).kind == "O" and dtype is not object:
+        err_msg = f"Passed {dtype=} is not a valid Pandas dtype."
+        if issubclass(dtype, datetime):
+            err_msg += ' To generate valid datetimes, pass `dtype="datetime64[ns]"`'
+            raise InvalidArgument(err_msg)
+        elif issubclass(dtype, timedelta):
+            err_msg += ' To generate valid timedeltas, pass `dtype="timedelta64[ns]"`'
+            raise InvalidArgument(err_msg)
         note_deprecation(
-            f"Passed dtype={dtype!r} is not a valid Pandas dtype.  We'll treat it as "
+            f"{err_msg}  We'll treat it as "
             "dtype=object for now, but this will be an error in a future version.",
             since="2021-12-31",
             has_codemod=False,
+            stacklevel=1,
         )
 
     if isinstance(dtype, st.SearchStrategy):
         raise InvalidArgument(
-            f"Passed dtype={dtype!r} is a strategy, but we require a concrete dtype "
+            f"Passed {dtype=} is a strategy, but we require a concrete dtype "
             "here.  See https://stackoverflow.com/q/74355937 for workaround patterns."
         )
-    dtype = try_convert(np.dtype, dtype, "dtype")
+
+    _get_subclasses = getattr(IntegerDtype, "__subclasses__", list)
+    dtype = {t.name: t() for t in _get_subclasses()}.get(dtype, dtype)
+
+    is_na_dtype = False
+    if isinstance(dtype, IntegerDtype):
+        is_na_dtype = True
+        dtype = np.dtype(dtype.name.lower())
+    elif dtype is not None:
+        dtype = try_convert(np.dtype, dtype, "dtype")
 
     if elements is None:
         elements = npst.from_dtype(dtype)
-    elif dtype is not None:
+        if is_na_dtype:
+            elements = st.none() | elements
+    # as an optimization, avoid converting object dtypes, which will always
+    # remain unchanged.
+    elif dtype is not None and dtype.kind != "O":
 
         def convert_element(value):
-            name = f"draw({prefix}elements)"
+            if is_na_dtype and value is None:
+                return None
+
             try:
                 return np.array([value], dtype=dtype)[0]
-            except TypeError:
+            except (TypeError, ValueError, OverflowError):
+                name = f"draw({prefix}elements)"
                 raise InvalidArgument(
-                    "Cannot convert %s=%r of type %s to dtype %s"
-                    % (name, value, type(value).__name__, dtype.str)
-                ) from None
-            except ValueError:
-                raise InvalidArgument(
-                    f"Cannot convert {name}={value!r} to type {dtype.str}"
+                    f"Cannot convert {name}={value!r} of type "
+                    f"{type(value).__name__} to dtype {dtype.str}"
                 ) from None
 
         elements = elements.map(convert_element)
@@ -165,8 +186,8 @@ DEFAULT_MAX_SIZE = 10
 @defines_strategy()
 def range_indexes(
     min_size: int = 0,
-    max_size: Optional[int] = None,
-    name: st.SearchStrategy[Optional[str]] = st.none(),
+    max_size: int | None = None,
+    name: st.SearchStrategy[str | None] = st.none(),
 ) -> st.SearchStrategy[pandas.RangeIndex]:
     """Provides a strategy which generates an :class:`~pandas.Index` whose
     values are 0, 1, ..., n for some n.
@@ -192,12 +213,12 @@ def range_indexes(
 @defines_strategy()
 def indexes(
     *,
-    elements: Optional[st.SearchStrategy[Ex]] = None,
+    elements: st.SearchStrategy[Ex] | None = None,
     dtype: Any = None,
     min_size: int = 0,
-    max_size: Optional[int] = None,
+    max_size: int | None = None,
     unique: bool = True,
-    name: st.SearchStrategy[Optional[str]] = st.none(),
+    name: st.SearchStrategy[str | None] = st.none(),
 ) -> st.SearchStrategy[pandas.Index]:
     """Provides a strategy for producing a :class:`pandas.Index`.
 
@@ -235,12 +256,17 @@ def indexes(
 @defines_strategy()
 def series(
     *,
-    elements: Optional[st.SearchStrategy[Ex]] = None,
+    elements: st.SearchStrategy[Ex] | None = None,
     dtype: Any = None,
-    index: Optional[st.SearchStrategy[Union[Sequence, pandas.Index]]] = None,
-    fill: Optional[st.SearchStrategy[Ex]] = None,
+    # new-style unions hit https://github.com/sphinx-doc/sphinx/issues/11211 during
+    # doc builds. See related comment in django/_fields.py. Quote to prevent
+    # shed/pyupgrade from changing it.
+    index: (
+        st.SearchStrategy["Union[Sequence, pandas.Index]"] | None  # noqa: UP007
+    ) = None,
+    fill: st.SearchStrategy[Ex] | None = None,
     unique: bool = False,
-    name: st.SearchStrategy[Optional[str]] = st.none(),
+    name: st.SearchStrategy[str | None] = st.none(),
 ) -> st.SearchStrategy[pandas.Series]:
     """Provides a strategy for producing a :class:`pandas.Series`.
 
@@ -283,9 +309,17 @@ def series(
     else:
         check_strategy(index, "index")
 
-    elements, dtype = elements_and_dtype(elements, dtype)
+    elements, np_dtype = elements_and_dtype(elements, dtype)
     index_strategy = index
 
+    # if it is converted to an object, use object for series type
+    if (
+        np_dtype is not None
+        and np_dtype.kind == "O"
+        and not isinstance(dtype, IntegerDtype)
+    ):
+        dtype = np_dtype
+
     @st.composite
     def result(draw):
         index = draw(index_strategy)
@@ -294,13 +328,13 @@ def series(
             if dtype is not None:
                 result_data = draw(
                     npst.arrays(
-                        dtype=dtype,
+                        dtype=object,
                         elements=elements,
                         shape=len(index),
                         fill=fill,
                         unique=unique,
                     )
-                )
+                ).tolist()
             else:
                 result_data = list(
                     draw(
@@ -311,25 +345,26 @@ def series(
                             fill=fill,
                             unique=unique,
                         )
-                    )
+                    ).tolist()
                 )
-
             return pandas.Series(result_data, index=index, dtype=dtype, name=draw(name))
         else:
             return pandas.Series(
                 (),
                 index=index,
-                dtype=dtype
-                if dtype is not None
-                else draw(dtype_for_elements_strategy(elements)),
+                dtype=(
+                    dtype
+                    if dtype is not None
+                    else draw(dtype_for_elements_strategy(elements))
+                ),
                 name=draw(name),
             )
 
     return result()
 
 
-@attr.s(slots=True)
-class column:
+@dataclass(slots=True, frozen=False)
+class column(Generic[Ex]):
     """Data object for describing a column in a DataFrame.
 
     Arguments:
@@ -346,21 +381,21 @@ class column:
     * unique: If all values in this column should be distinct.
     """
 
-    name = attr.ib(default=None)
-    elements = attr.ib(default=None)
-    dtype = attr.ib(default=None, repr=get_pretty_function_description)
-    fill = attr.ib(default=None)
-    unique = attr.ib(default=False)
+    name: str | int | None = None
+    elements: st.SearchStrategy[Ex] | None = None
+    dtype: Any = None
+    fill: st.SearchStrategy[Ex] | None = None
+    unique: bool = False
 
 
 def columns(
-    names_or_number: Union[int, Sequence[str]],
+    names_or_number: int | Sequence[str],
     *,
     dtype: Any = None,
-    elements: Optional[st.SearchStrategy[Ex]] = None,
-    fill: Optional[st.SearchStrategy[Ex]] = None,
+    elements: st.SearchStrategy[Ex] | None = None,
+    fill: st.SearchStrategy[Ex] | None = None,
     unique: bool = False,
-) -> List[column]:
+) -> list[column[Ex]]:
     """A convenience function for producing a list of :class:`column` objects
     of the same general shape.
 
@@ -371,7 +406,7 @@ def columns(
     create the columns.
     """
     if isinstance(names_or_number, (int, float)):
-        names: List[Union[int, str, None]] = [None] * names_or_number
+        names: list[int | str | None] = [None] * names_or_number
     else:
         names = list(names_or_number)
     return [
@@ -382,10 +417,10 @@ def columns(
 
 @defines_strategy()
 def data_frames(
-    columns: Optional[Sequence[column]] = None,
+    columns: Sequence[column] | None = None,
     *,
-    rows: Optional[st.SearchStrategy[Union[dict, Sequence[Any]]]] = None,
-    index: Optional[st.SearchStrategy[Ex]] = None,
+    rows: st.SearchStrategy[dict | Sequence[Any]] | None = None,
+    index: st.SearchStrategy[Ex] | None = None,
 ) -> st.SearchStrategy[pandas.DataFrame]:
     """Provides a strategy for producing a :class:`pandas.DataFrame`.
 
@@ -505,7 +540,6 @@ def data_frames(
             def rows_only(draw):
                 index = draw(index_strategy)
 
-                @check_function
                 def row():
                     result = draw(rows)
                     check_type(abc.Iterable, result, "draw(row)")
@@ -526,7 +560,7 @@ def data_frames(
     cols = try_convert(tuple, columns, "columns")
 
     rewritten_columns = []
-    column_names: Set[str] = set()
+    column_names: set[str] = set()
 
     for i, c in enumerate(cols):
         check_type(column, c, f"columns[{i}]")
@@ -549,8 +583,7 @@ def data_frames(
             raise InvalidArgument(f"duplicate definition of column name {c.name!r}")
 
         column_names.add(c.name)
-
-        c.elements, c.dtype = elements_and_dtype(c.elements, c.dtype, label)
+        c.elements, _ = elements_and_dtype(c.elements, c.dtype, label)
 
         if c.dtype is None and rows is not None:
             raise InvalidArgument(
@@ -560,7 +593,6 @@ def data_frames(
         c.fill = npst.fill_for(
             fill=c.fill, elements=c.elements, unique=c.unique, name=label
         )
-
         rewritten_columns.append(c)
 
     if rows is None:
@@ -580,17 +612,18 @@ def data_frames(
 
             # For columns with no filling the problem is harder, and drawing
             # them like that would result in rows being very far apart from
-            # each other in the underlying data stream, which gets in the way
+            # each other in the choice sequence, which gets in the way
             # of shrinking. So what we do is reorder and draw those columns
             # row wise, so that the values of each row are next to each other.
-            # This makes life easier for the shrinker when deleting blocks of
-            # data.
-            columns_without_fill = [c for c in rewritten_columns if c.fill.is_empty]
+            # This makes life easier for the shrinker when deleting choices.
 
+            columns_without_fill = [c for c in rewritten_columns if c.fill.is_empty]
             if columns_without_fill:
                 for c in columns_without_fill:
                     data[c.name] = pandas.Series(
-                        np.zeros(shape=len(index), dtype=c.dtype), index=index
+                        np.zeros(shape=len(index), dtype=object),
+                        index=index,
+                        dtype=c.dtype,
                     )
                 seen = {c.name: set() for c in columns_without_fill if c.unique}
 
@@ -606,8 +639,9 @@ def data_frames(
                                 reject()
                         else:
                             value = draw(c.elements)
+
                         try:
-                            data[c.name][i] = value
+                            data[c.name].iloc[i] = value
                         except ValueError as err:  # pragma: no cover
                             # This just works in Pandas 1.4 and later, but gives
                             # a confusing error on previous versions.
@@ -615,7 +649,7 @@ def data_frames(
                                 value, (float, int, str, bool, datetime, timedelta)
                             ):
                                 raise ValueError(
-                                    f"Failed to add value={value!r} to column "
+                                    f"Failed to add {value=} to column "
                                     f"{c.name} with dtype=None.  Maybe passing "
                                     "dtype=object would help?"
                                 ) from err
@@ -688,13 +722,13 @@ def data_frames(
                         for k in row:
                             if k not in column_names:
                                 raise InvalidArgument(
-                                    "Row %r contains column %r not in columns %r)"
-                                    % (row, k, [c.name for c in rewritten_columns])
+                                    f"Row {row!r} contains column {k!r} not in "
+                                    f"columns {[c.name for c in rewritten_columns]!r})"
                                 )
                         row = as_list
                     if any_unique:
                         has_duplicate = False
-                        for seen, value in zip(all_seen, row):
+                        for seen, value in zip(all_seen, row, strict=False):
                             if seen is None:
                                 continue
                             if value in seen:
